@@ -18,13 +18,16 @@
 Provides XML I{element} classes.
 """
 
-from suds import *
-from suds.sax import *
+from logging import getLogger
+from suds.compat import unicode, basestring
+from suds.sax import Namespace, splitPrefix
 from suds.sax.text import Text
 from suds.sax.attribute import Attribute
 
+log = getLogger(__name__)
 
-class Element(UnicodeMixin):
+
+class Element:
     """
     An XML element object.
     @ivar parent: The node containing this attribute
@@ -48,18 +51,21 @@ class Element(UnicodeMixin):
     """
 
     matcher = {
-        'eq': lambda a,b: a == b,
-        'startswith' : lambda a,b: a.startswith(b),
-        'endswith' : lambda a,b: a.endswith(b),
-        'contains' : lambda a,b: b in a}
+        'eq': lambda a, b: a == b,
+        'startswith': lambda a, b: a.startswith(b),
+        'endswith': lambda a, b: a.endswith(b),
+        'contains': lambda a, b: b in a
+    }
 
-    specialprefixes = {Namespace.xmlns[0] : Namespace.xmlns[1]}
+    specialprefixes = {
+        Namespace.xmlns[0]: Namespace.xmlns[1]
+    }
 
     @classmethod
     def buildPath(self, parent, path):
-        """Build the specifed path as a/b/c.
-
-        Any missing intermediate nodes are built automatically.
+        """
+        Build the specifed pat as a/b/c where missing intermediate nodes are
+        built automatically.
         @param parent: A parent element on which the path is built.
         @type parent: I{Element}
         @param path: A simple path separated by (/).
@@ -76,13 +82,14 @@ class Element(UnicodeMixin):
 
     def __init__(self, name, parent=None, ns=None):
         """
-        @param name: The element's (tag) name. May contain a prefix.
+        @param name: The element's (tag) name.  May cotain a prefix.
         @type name: basestring
         @param parent: An optional parent element.
         @type parent: I{Element}
-        @param ns: An optional namespace.
+        @param ns: An optional namespace
         @type ns: (I{prefix}, I{name})
         """
+
         self.rename(name)
         self.expns = None
         self.nsprefixes = {}
@@ -121,7 +128,6 @@ class Element(UnicodeMixin):
         """
         self.prefix = p
         if p is not None and u is not None:
-            self.expns = None
             self.addPrefix(p, u)
         return self
 
@@ -133,7 +139,8 @@ class Element(UnicodeMixin):
         """
         if self.prefix is None:
             return self.name
-        return '%s:%s' % (self.prefix, self.name)
+        else:
+            return '%s:%s' % (self.prefix, self.name)
 
     def getRoot(self):
         """
@@ -143,7 +150,8 @@ class Element(UnicodeMixin):
         """
         if self.parent is None:
             return self
-        return self.parent.getRoot()
+        else:
+            return self.parent.getRoot()
 
     def clone(self, parent=None):
         """
@@ -158,7 +166,7 @@ class Element(UnicodeMixin):
             root.append(a.clone(self))
         for c in self.children:
             root.append(c.clone(self))
-        for item in list(self.nsprefixes.items()):
+        for item in self.nsprefixes.items():
             root.addPrefix(item[0], item[1])
         return root
 
@@ -223,7 +231,8 @@ class Element(UnicodeMixin):
         attr = self.getAttribute(name, ns)
         if attr is None or attr.value is None:
             return default
-        return attr.getValue()
+        else:
+            return attr.getValue()
 
     def setText(self, value):
         """
@@ -249,7 +258,8 @@ class Element(UnicodeMixin):
         """
         if self.hasText():
             return self.text
-        return default
+        else:
+            return default
 
     def trim(self):
         """
@@ -279,7 +289,8 @@ class Element(UnicodeMixin):
         """
         if self.prefix is None:
             return self.defaultNamespace()
-        return self.resolvePrefix(self.prefix)
+        else:
+            return self.resolvePrefix(self.prefix)
 
     def defaultNamespace(self):
         """
@@ -292,14 +303,15 @@ class Element(UnicodeMixin):
         p = self
         while p is not None:
             if p.expns is not None:
-                return None, p.expns
-            p = p.parent
+                return (None, p.expns)
+            else:
+                p = p.parent
         return Namespace.default
 
     def append(self, objects):
         """
-        Append the specified child based on whether it is an element or an
-        attribute.
+        Append the specified child based on whether it is an
+        element or an attrbuite.
         @param objects: A (single|collection) of attribute(s) or element(s)
             to be added as children.
         @type objects: (L{Element}|L{Attribute})
@@ -323,8 +335,8 @@ class Element(UnicodeMixin):
     def insert(self, objects, index=0):
         """
         Insert an L{Element} content at the specified index.
-        @param objects: A (single|collection) of attribute(s) or element(s) to
-            be added as children.
+        @param objects: A (single|collection) of attribute(s) or element(s)
+            to be added as children.
         @type objects: (L{Element}|L{Attribute})
         @param index: The position in the list of children to insert.
         @type index: int
@@ -437,7 +449,7 @@ class Element(UnicodeMixin):
                 ns = node.resolvePrefix(prefix)
             result = node.getChild(name, ns)
             if result is None:
-                break;
+                break
             else:
                 node = result
         return result
@@ -507,9 +519,9 @@ class Element(UnicodeMixin):
         n = self
         while n is not None:
             if prefix in n.nsprefixes:
-                return prefix, n.nsprefixes[prefix]
+                return (prefix, n.nsprefixes[prefix])
             if prefix in self.specialprefixes:
-                return prefix, self.specialprefixes[prefix]
+                return (prefix, self.specialprefixes[prefix])
             n = n.parent
         return default
 
@@ -567,23 +579,24 @@ class Element(UnicodeMixin):
         @return: A mapped prefix.
         @rtype: basestring
         """
-        for item in list(self.nsprefixes.items()):
+        for item in self.nsprefixes.items():
             if item[1] == uri:
                 prefix = item[0]
                 return prefix
-        for item in list(self.specialprefixes.items()):
+        for item in self.specialprefixes.items():
             if item[1] == uri:
                 prefix = item[0]
                 return prefix
         if self.parent is not None:
             return self.parent.findPrefix(uri, default)
-        return default
+        else:
+            return default
 
     def findPrefixes(self, uri, match='eq'):
         """
-        Find all prefixes that have been mapped to a namespace URI.
-        The local mapping is searched, then it walks up the tree until it
-        reaches the top, collecting all matches.
+        Find all prefixes that has been mapped to a namespace URI.
+        The local mapping is searched, then it walks up the tree until
+        it reaches the top collecting all matches.
         @param uri: A namespace URI.
         @type uri: basestring
         @param match: A matching function L{Element.matcher}.
@@ -592,11 +605,11 @@ class Element(UnicodeMixin):
         @rtype: [basestring,...]
         """
         result = []
-        for item in list(self.nsprefixes.items()):
+        for item in self.nsprefixes.items():
             if self.matcher[match](item[1], uri):
                 prefix = item[0]
                 result.append(prefix)
-        for item in list(self.specialprefixes.items()):
+        for item in self.specialprefixes.items():
             if self.matcher[match](item[1], uri):
                 prefix = item[0]
                 result.append(prefix)
@@ -617,15 +630,20 @@ class Element(UnicodeMixin):
             c.promotePrefixes()
         if self.parent is None:
             return
-        for p,u in list(self.nsprefixes.items()):
+        _pref = []
+        for p, u in self.nsprefixes.items():
             if p in self.parent.nsprefixes:
                 pu = self.parent.nsprefixes[p]
                 if pu == u:
-                    del self.nsprefixes[p]
+                    _pref.append(p)
                 continue
             if p != self.parent.prefix:
                 self.parent.nsprefixes[p] = u
-                del self.nsprefixes[p]
+                _pref.append(p)
+
+        for x in _pref:
+            del self.nsprefixes[x]
+
         return self
 
     def refitPrefixes(self):
@@ -649,8 +667,8 @@ class Element(UnicodeMixin):
         """
         Normalize the namespace prefixes.
         This generates unique prefixes for all namespaces.  Then retrofits all
-        prefixes and prefix mappings.  Further, it will retrofix attribute values
-        that have values containing (:).
+        prefixes and prefix mappings.  Further, it will retrofix attribute
+        values that have values containing (:).
         @return: self
         @rtype: L{Element}
         """
@@ -667,11 +685,12 @@ class Element(UnicodeMixin):
         """
         noattrs = not len(self.attributes)
         nochildren = not len(self.children)
-        notext = ( self.text is None )
-        nocontent = ( nochildren and notext )
+        notext = self.text is None
+        nocontent = nochildren and notext
         if content:
             return nocontent
-        return nocontent and noattrs
+        else:
+            return nocontent and noattrs
 
     def isnil(self):
         """
@@ -681,19 +700,22 @@ class Element(UnicodeMixin):
         @rtype: boolean
         """
         nilattr = self.getAttribute('nil', ns=Namespace.xsins)
-        return nilattr is not None and ( nilattr.getValue().lower() == 'true' )
+        if nilattr is None:
+            return False
+        else:
+            return nilattr.getValue().lower() == 'true'
 
     def setnil(self, flag=True):
         """
         Set this node to I{nil} as defined by having an
         attribute I{xsi:nil}=I{flag}.
-        @param flag: A flag indicating how I{xsi:nil} will be set.
+        @param flag: A flag inidcating how I{xsi:nil} will be set.
         @type flag: boolean
         @return: self
         @rtype: L{Element}
         """
         p, u = Namespace.xsins
-        name  = ':'.join((p, 'nil'))
+        name = ':'.join((p, 'nil'))
         self.set(name, str(flag).lower())
         self.addPrefix(p, u)
         if flag:
@@ -726,11 +748,11 @@ class Element(UnicodeMixin):
         @return: A I{pretty} string.
         @rtype: basestring
         """
-        tab = '%*s'%(indent*3,'')
+        tab = '%*s' % (indent * 3, '')
         result = []
         result.append('%s<%s' % (tab, self.qname()))
         result.append(self.nsdeclarations())
-        for a in [str(a) for a in self.attributes]:
+        for a in [unicode(a) for a in self.attributes]:
             result.append(' %s' % a)
         if self.isempty():
             result.append('/>')
@@ -744,7 +766,8 @@ class Element(UnicodeMixin):
         if len(self.children):
             result.append('\n%s' % tab)
         result.append('</%s>' % self.qname())
-        return ''.join(result)
+        result = ''.join(result)
+        return result
 
     def plain(self):
         """
@@ -755,7 +778,7 @@ class Element(UnicodeMixin):
         result = []
         result.append('<%s' % self.qname())
         result.append(self.nsdeclarations())
-        for a in [str(a) for a in self.attributes]:
+        for a in [unicode(a) for a in self.attributes]:
             result.append(' %s' % a)
         if self.isempty():
             result.append('/>')
@@ -766,7 +789,8 @@ class Element(UnicodeMixin):
         for c in self.children:
             result.append(c.plain())
         result.append('</%s>' % self.qname())
-        return ''.join(result)
+        result = ''.join(result)
+        return result
 
     def nsdeclarations(self):
         """
@@ -785,11 +809,11 @@ class Element(UnicodeMixin):
             if self.expns is not None:
                 d = ' xmlns="%s"' % self.expns
                 s.append(d)
-        for item in list(self.nsprefixes.items()):
-            (p,u) = item
+        for p, u in self.nsprefixes.items():
             if self.parent is not None:
                 ns = self.parent.resolvePrefix(p)
-                if ns[1] == u: continue
+                if ns[1] == u:
+                    continue
             d = ' xmlns:%s="%s"' % (p, u)
             s.append(d)
         return ''.join(s)
@@ -804,8 +828,14 @@ class Element(UnicodeMixin):
         @return: True if matched.
         @rtype: boolean
         """
-        byname = name is None or ( self.name == name )
-        byns = ns is None or ( self.namespace()[1] == ns[1] )
+        if name is None:
+            byname = True
+        else:
+            byname = self.name == name
+        if ns is None:
+            byns = True
+        else:
+            byns = self.namespace()[1] == ns[1]
         return byname and byns
 
     def branch(self):
@@ -834,7 +864,8 @@ class Element(UnicodeMixin):
 
     def walk(self, visitor):
         """
-        Walk the branch and call the visitor function on each node.
+        Walk the branch and call the visitor function
+        on each node.
         @param visitor: A function.
         @return: self
         @rtype: L{Element}
@@ -884,24 +915,33 @@ class Element(UnicodeMixin):
         return len(self.children)
 
     def __getitem__(self, index):
-        if isinstance(index, str):
+        if isinstance(index, basestring):
             return self.get(index)
-        if index < len(self.children):
-            return self.children[index]
+        else:
+            if index < len(self.children):
+                return self.children[index]
+            else:
+                return None
 
     def __setitem__(self, index, value):
-        if isinstance(index, str):
+        if isinstance(index, basestring):
             self.set(index, value)
         else:
             if index < len(self.children) and isinstance(value, Element):
                 self.children.insert(index, value)
 
     def __eq__(self, rhs):
-        return isinstance(rhs, Element) and  \
-            self.match(rhs.name, rhs.namespace())
+        return rhs is not None and \
+            isinstance(rhs, Element) and \
+            self.name == rhs.name and \
+            self.namespace()[1] == rhs.namespace()[1]
 
     def __repr__(self):
-        return 'Element (prefix=%s, name=%s)' % (self.prefix, self.name)
+        return \
+            'Element (prefix=%s, name=%s)' % (self.prefix, self.name)
+
+    def __str__(self):
+        return self.str()
 
     def __unicode__(self):
         return self.str()
@@ -928,6 +968,9 @@ class NodeIterator:
         self.children = parent.children
 
     def __next__(self):
+        return self.next()
+
+    def next(self):
         """
         Get the next child.
         @return: The next child.
@@ -999,7 +1042,7 @@ class PrefixNormalizer:
         @rtype: set
         """
         s = set()
-        for ns in list(n.nsprefixes.items()):
+        for ns in n.nsprefixes.items():
             if self.permit(ns):
                 s.add(ns[1])
         return s
@@ -1063,8 +1106,9 @@ class PrefixNormalizer:
         @param a: An attribute.
         @type a: L{Attribute}
         """
-        p,name = splitPrefix(a.getValue())
-        if p is None: return
+        p, name = splitPrefix(a.getValue())
+        if p is None:
+            return
         ns = a.resolvePrefix(p)
         if self.permit(ns):
             u = ns[1]
@@ -1078,7 +1122,7 @@ class PrefixNormalizer:
         for n in self.branch:
             n.nsprefixes = {}
         n = self.node
-        for u, p in list(self.prefixes.items()):
+        for u, p in self.prefixes.items():
             n.addPrefix(p, u)
 
     def permit(self, ns):
@@ -1099,8 +1143,4 @@ class PrefixNormalizer:
         @return: True if to be skipped.
         @rtype: boolean
         """
-        return (ns is None or
-            ns == Namespace.default or
-            ns == Namespace.xsdns or
-            ns == Namespace.xsins or
-            ns == Namespace.xmlns)
+        return ns in (None, Namespace.default, Namespace.xsdns, Namespace.xsins, Namespace.xmlns)
