@@ -15,29 +15,32 @@
 # written by: Jeff Ortel ( jortel@redhat.com )
 
 """
-The I{sxbase} module provides I{base} classes representing schema objects.
+The I{sxbase} module provides I{base} classes that represent
+schema objects.
 """
 
-from suds import *
-from suds.xsd import *
+from logging import getLogger
+from suds import objid, Repr
+from suds.compat import unicode
+from suds.xsd import isqref, Filter, qualify
 from suds.sax.element import Element
 from suds.sax import Namespace
 
-from logging import getLogger
 log = getLogger(__name__)
 
 
-class SchemaObject(UnicodeMixin):
+class SchemaObject(object):
     """
-    A schema object is an extension to object with schema awareness.
+    A schema object is an extension to object object with
+    with schema awareness.
     @ivar root: The XML root element.
     @type root: L{Element}
     @ivar schema: The schema containing this object.
     @type schema: L{schema.Schema}
-    @ivar form_qualified: A flag that indicates that @elementFormDefault
+    @ivar form_qualified: A flag that inidcates that @elementFormDefault
         has a value of I{qualified}.
     @type form_qualified: boolean
-    @ivar nillable: A flag that indicates that @nillable
+    @ivar nillable: A flag that inidcates that @nillable
         has a value of I{true}.
     @type nillable: boolean
     @ivar default: The default value.
@@ -49,7 +52,7 @@ class SchemaObject(UnicodeMixin):
     @classmethod
     def prepend(cls, d, s, filter=Filter()):
         """
-        Prepend schema objects from B{s}ource list to
+        Prepend schema object's from B{s}ource list to
         the B{d}estination list while applying the filter.
         @param d: The destination list.
         @type d: list
@@ -67,7 +70,7 @@ class SchemaObject(UnicodeMixin):
     @classmethod
     def append(cls, d, s, filter=Filter()):
         """
-        Append schema objects from B{s}ource list to
+        Append schema object's from B{s}ource list to
         the B{d}estination list while applying the filter.
         @param d: The destination list.
         @type d: list
@@ -84,7 +87,7 @@ class SchemaObject(UnicodeMixin):
         """
         @param schema: The containing schema.
         @type schema: L{schema.Schema}
-        @param root: The XML root node.
+        @param root: The xml root node.
         @type root: L{Element}
         """
         self.schema = schema
@@ -100,6 +103,7 @@ class SchemaObject(UnicodeMixin):
         self.nillable = False
         self.default = root.get('default')
         self.rawchildren = []
+        self.cache = {}
 
     def attributes(self, filter=Filter()):
         """
@@ -131,7 +135,7 @@ class SchemaObject(UnicodeMixin):
 
     def get_attribute(self, name):
         """
-        Get (find) an attribute by name.
+        Get (find) a I{non-attribute} attribute by name.
         @param name: A attribute name.
         @type name: str
         @return: A tuple: the requested (attribute, ancestry).
@@ -139,8 +143,8 @@ class SchemaObject(UnicodeMixin):
         """
         for child, ancestry in self.attributes():
             if child.name == name:
-                return child, ancestry
-        return None, []
+                return (child, ancestry)
+        return (None, [])
 
     def get_child(self, name):
         """
@@ -152,12 +156,12 @@ class SchemaObject(UnicodeMixin):
         """
         for child, ancestry in self.children():
             if child.any() or child.name == name:
-                return child, ancestry
-        return None, []
+                return (child, ancestry)
+        return (None, [])
 
     def namespace(self, prefix=None):
         """
-        Get this property's namespace.
+        Get this properties namespace
         @param prefix: The default prefix.
         @type prefix: str
         @return: The schema's target namespace
@@ -171,18 +175,19 @@ class SchemaObject(UnicodeMixin):
     def default_namespace(self):
         return self.root.defaultNamespace()
 
-    def multi_occurrence(self):
+    def unbounded(self):
         """
-        Get whether the node has multiple occurrences, i.e. is a I{collection}.
-        @return: True if it has, False if it has 1 occurrence at most.
+        Get whether this node is unbounded I{(a collection)}
+        @return: True if unbounded, else False.
         @rtype: boolean
         """
         max = self.max
         if max is None:
-            return False
+            max = '1'
         if max.isdigit():
-            return int(max) > 1
-        return max == 'unbounded'
+            return (int(max) > 1)
+        else:
+            return max == 'unbounded'
 
     def optional(self):
         """
@@ -190,7 +195,10 @@ class SchemaObject(UnicodeMixin):
         @return: True if optional, else False
         @rtype: boolean
         """
-        return self.min == '0'
+        min = self.min
+        if min is None:
+            min = '1'
+        return min == '0'
 
     def required(self):
         """
@@ -202,20 +210,17 @@ class SchemaObject(UnicodeMixin):
 
     def resolve(self, nobuiltin=False):
         """
-        Resolve the node's type reference and return the referenced type node.
-
-        Only schema objects that actually support 'having a type' custom
-        implement this interface while others simply resolve as themselves.
-        @param nobuiltin: Flag indicating whether resolving to an external XSD
-            builtin type should not be allowed.
+        Resolve and return the nodes true self.
+        @param nobuiltin: Flag indicates that resolution must
+            not continue to include xsd builtins.
         @return: The resolved (true) type.
         @rtype: L{SchemaObject}
         """
-        return self
+        return self.cache.get(nobuiltin, self)
 
     def sequence(self):
         """
-        Get whether this is a <xs:sequence/>.
+        Get whether this is an <xs:sequence/>
         @return: True if <xs:sequence/>, else False
         @rtype: boolean
         """
@@ -223,7 +228,7 @@ class SchemaObject(UnicodeMixin):
 
     def xslist(self):
         """
-        Get whether this is a <xs:list/>.
+        Get whether this is an <xs:list/>
         @return: True if any, else False
         @rtype: boolean
         """
@@ -231,7 +236,7 @@ class SchemaObject(UnicodeMixin):
 
     def all(self):
         """
-        Get whether this is an <xs:all/>.
+        Get whether this is an <xs:all/>
         @return: True if any, else False
         @rtype: boolean
         """
@@ -239,7 +244,7 @@ class SchemaObject(UnicodeMixin):
 
     def choice(self):
         """
-        Get whether this is a <xs:choice/>.
+        Get whether this is n <xs:choice/>
         @return: True if any, else False
         @rtype: boolean
         """
@@ -247,7 +252,7 @@ class SchemaObject(UnicodeMixin):
 
     def any(self):
         """
-        Get whether this is an <xs:any/>.
+        Get whether this is an <xs:any/>
         @return: True if any, else False
         @rtype: boolean
         """
@@ -322,7 +327,7 @@ class SchemaObject(UnicodeMixin):
 
     def translate(self, value, topython=True):
         """
-        Translate a value (type) to/from a Python type.
+        Translate a value (type) to/from a python type.
         @param value: A value to translate.
         @return: The converted I{language} type.
         """
@@ -338,11 +343,11 @@ class SchemaObject(UnicodeMixin):
 
     def dependencies(self):
         """
-        Get a list of dependencies for dereferencing.
-        @return: A merge dependency index and a list of dependencies.
+        Get a list of dependancies for dereferencing.
+        @return: A merge dependancy index and a list of dependancies.
         @rtype: (int, [L{SchemaObject},...])
         """
-        return None, []
+        return (None, [])
 
     def autoqualified(self):
         """
@@ -356,9 +361,9 @@ class SchemaObject(UnicodeMixin):
     def qualify(self):
         """
         Convert attribute values, that are references to other
-        objects, into I{qref}.  Qualified using the default document namespace.
-        Since many WSDLs are written improperly: when the document does
-        not define its default namespace, the schema target namespace is used
+        objects, into I{qref}.  Qualfied using default document namespace.
+        Since many wsdls are written improperly: when the document does
+        not define a default namespace, the schema target namespace is used
         to qualify references.
         """
         defns = self.root.defaultNamespace()
@@ -386,7 +391,7 @@ class SchemaObject(UnicodeMixin):
                   'default',
                   'type',
                   'nillable',
-                  'form_qualified'):
+                  'form_qualified',):
             if getattr(self, n) is not None:
                 continue
             v = getattr(other, n)
@@ -396,7 +401,7 @@ class SchemaObject(UnicodeMixin):
 
     def content(self, collection=None, filter=Filter(), history=None):
         """
-        Get a I{flattened} list of this node's contents.
+        Get a I{flattened} list of this nodes contents.
         @param collection: A list to fill.
         @type collection: list
         @param filter: A filter used to constrain the result.
@@ -432,8 +437,9 @@ class SchemaObject(UnicodeMixin):
         if self in history:
             return '%s ...' % Repr(self)
         history.append(self)
-        tab = '%*s'%(indent*3, '')
-        result = ['%s<%s' % (tab, self.id)]
+        tab = '%*s' % (indent * 3, '')
+        result = []
+        result.append('%s<%s' % (tab, self.id))
         for n in self.description():
             if not hasattr(self, n):
                 continue
@@ -457,13 +463,16 @@ class SchemaObject(UnicodeMixin):
     def description(self):
         """
         Get the names used for str() and repr() description.
-        @return:  A dictionary of relevant attributes.
+        @return:  A dictionary of relavent attributes.
         @rtype: [str,...]
         """
         return ()
 
+    def __str__(self):
+        return self.__repr__()
+
     def __unicode__(self):
-        return str(self.str())
+         return self.__repr__()
 
     def __repr__(self):
         s = []
@@ -476,27 +485,23 @@ class SchemaObject(UnicodeMixin):
                 continue
             s.append(' %s="%s"' % (n, v))
         s.append(' />')
-        return ''.join(s)
+        myrep = ''.join(s)
+        return myrep
 
     def __len__(self):
         n = 0
-        for x in self: n += 1
+        for x in self:
+            n += 1
         return n
 
     def __iter__(self):
         return Iter(self)
 
     def __getitem__(self, index):
-        """Returns a contained schema object referenced by its 0-based index.
-
-        Returns None if such an object does not exist.
-
-        """
         i = 0
         for c in self:
             if i == index:
                 return c
-            i += 1
 
 
 class Iter:
@@ -520,7 +525,7 @@ class Iter:
             self.items = sx.rawchildren
             self.index = 0
 
-        def __next__(self):
+        def next(self):
             """
             Get the I{next} item in the frame's collection.
             @return: The next item or None
@@ -572,6 +577,9 @@ class Iter:
             raise StopIteration()
 
     def __next__(self):
+        return self.next()
+
+    def next(self):
         """
         Get the next item.
         @return: A tuple: the next (child, ancestry).
@@ -580,15 +588,15 @@ class Iter:
         """
         frame = self.top()
         while True:
-            result = next(frame)
+            result = frame.next()
             if result is None:
                 self.pop()
-                return next(self)
+                return self.next()
             if isinstance(result, Content):
                 ancestry = [f.sx for f in self.stack]
-                return result, ancestry
+                return (result, ancestry)
             self.push(result)
-            return next(self)
+            return self.next()
 
     def __iter__(self):
         return self
@@ -596,7 +604,7 @@ class Iter:
 
 class XBuiltin(SchemaObject):
     """
-    Represents an (XSD) schema <xs:*/> node.
+    Represents an (xsd) schema <xs:*/> node
     """
 
     def __init__(self, schema, name):
@@ -615,6 +623,9 @@ class XBuiltin(SchemaObject):
     def builtin(self):
         return True
 
+    def resolve(self, nobuiltin=False):
+        return self
+
 
 class Content(SchemaObject):
     """
@@ -626,7 +637,7 @@ class Content(SchemaObject):
 
 class NodeFinder:
     """
-    Find nodes based on flexable criteria.  The I{matcher}
+    Find nodes based on flexable criteria.  The I{matcher} is
     may be any object that implements a match(n) method.
     @ivar matcher: An object used as criteria for match.
     @type matcher: I{any}.match(n)
